@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -125,8 +125,6 @@ g_mime_multipart_signed_init (GMimeMultipartSigned *mps, GMimeMultipartSignedCla
 static void
 g_mime_multipart_signed_finalize (GObject *object)
 {
-	GMimeMultipartSigned *mps = (GMimeMultipartSigned *) object;
-	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -410,8 +408,12 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	
 	if (protocol) {
 		/* make sure the protocol matches the cipher sign protocol */
-		if (g_ascii_strcasecmp (ctx->sign_protocol, protocol) != 0)
+		if (g_ascii_strcasecmp (ctx->sign_protocol, protocol) != 0) {
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     "Cannot verify multipart/signed part: unsupported signature protocol '%s'.",
+				     protocol);
 			return NULL;
+		}
 	} else {
 		/* *shrug* - I guess just go on as if they match? */
 		protocol = ctx->sign_protocol;
@@ -422,6 +424,8 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
+			     "Cannot verify multipart/signed part: signature content-type does not match protocol.");
 		g_free (content_type);
 		
 		return NULL;
