@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -141,11 +141,11 @@ decode_quoted_string (const char **in)
 	
 	if (*inptr == '"') {
 		start++;
-		out = g_strndup (start, (unsigned int) (inptr - start));
+		out = g_strndup (start, (size_t) (inptr - start));
 		inptr++;
 	} else {
 		/* string wasn't properly quoted */
-		out = g_strndup (start, (unsigned int) (inptr - start));
+		out = g_strndup (start, (size_t) (inptr - start));
 	}
 	
 	*in = inptr;
@@ -173,11 +173,24 @@ decode_token (const char **in)
 	decode_lwsp (&inptr);
 	
 	start = inptr;
+#ifdef STRICT_PARSER
 	while (is_ttoken (*inptr))
 		inptr++;
+#else
+	/* Broken mail clients like to make our lives difficult. Scan
+	 * for a ';' instead of trusting that the client followed the
+	 * specification. */
+	while (*inptr && *inptr != ';')
+		inptr++;
+	
+	/* Scan backwards over any trailing lwsp */
+	while (inptr > start && is_lwsp (inptr[-1]))
+		inptr--;
+#endif
+	
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned int) (inptr - start));
+		return g_strndup (start, (size_t) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -215,7 +228,7 @@ decode_param_token (const char **in)
 		inptr++;
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned int) (inptr - start));
+		return g_strndup (start, (size_t) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -396,7 +409,7 @@ rfc2184_param_charset (const char **in, char **langp)
 	
 	if (*inptr == '\'') {
 		if (langp)
-			*langp = g_strndup (lang, inptr - lang);
+			*langp = g_strndup (lang, (size_t) (inptr - lang));
 		
 		inptr++;
 	}
@@ -517,7 +530,8 @@ decode_param_list (const char *in)
 	char *name, *value;
 	gboolean encoded;
 	GString *gvalue;
-	int id, i;
+	guint i;
+	int id;
 	
 	params = NULL;
 	tail = (GMimeParam *) &params;
@@ -855,8 +869,8 @@ param_list_format (GString *out, const GMimeParam *param, gboolean fold)
 	
 	while (param) {
 		gboolean encoded = FALSE;
-		unsigned nlen, vlen;
 		int here = out->len;
+		size_t nlen, vlen;
 		int quote = 0;
 		char *value;
 		
@@ -895,7 +909,7 @@ param_list_format (GString *out, const GMimeParam *param, gboolean fold)
 		
 		if (nlen + vlen + quote > GMIME_FOLD_LEN - 2) {
 			/* we need to do special rfc2184 parameter wrapping */
-			int maxlen = GMIME_FOLD_LEN - (nlen + 6);
+			size_t maxlen = GMIME_FOLD_LEN - (nlen + 6);
 			char *inptr, *inend;
 			int i = 0;
 			
@@ -903,7 +917,7 @@ param_list_format (GString *out, const GMimeParam *param, gboolean fold)
 			inend = value + vlen;
 			
 			while (inptr < inend) {
-				char *ptr = inptr + MIN (inend - inptr, maxlen);
+				char *ptr = inptr + MIN ((size_t) (inend - inptr), maxlen);
 				
 				if (encoded && ptr < inend) {
 					/* be careful not to break an encoded char (ie %20) */
@@ -929,9 +943,9 @@ param_list_format (GString *out, const GMimeParam *param, gboolean fold)
 							i++, encoded ? "*" : "");
 				
 				if (encoded || !quote)
-					g_string_append_len (out, inptr, ptr - inptr);
+					g_string_append_len (out, inptr, (size_t) (ptr - inptr));
 				else
-					g_string_append_len_quoted (out, inptr, ptr - inptr);
+					g_string_append_len_quoted (out, inptr, (size_t) (ptr - inptr));
 				
 				used += (out->len - here);
 				
